{% extends "ox_layout.html" %}
{% block body %}


<div class="w3-teal w3-margin-bottom w3-panel">
  <h2>Available Plugins</h2>
  <p>
    Click on the name of a plugin to expand and use its components.
  </p>
</div>

{% for plug_name, comp_list in components %}
<div class="w3-light-grey w3-margin-bottom w3-row-padding">  
  <details>
    <summary style="font-size:200%;">{{ plug_name }}</summary>
    <UL>
      {% for cmd, url in comp_list %}
      <LI><A HREF="{{url}}">{{cmd}}</A></LI>
      {% endfor %}
    </UL>
    </details>
</div>
{% endfor %}

{% endblock %}
