
{% set oh_task = item.kwargs['ox_herd_task'] %}
<details>
  <summary>{{ oh_task.name }} : queue = {{ oh_task.queue_name }} {{ oh_task.manager }} </summary>
  <TABLE cellpadding="10"><TR>
	<TH>
	  <A HREF="{{url_for('ox_herd.launch_job')+'?jid=' + item.id}}">
	    Launch Job Copy</A>
	</TH>
	<TH>
	  <A HREF="{{url_for('ox_herd.schedule_job')+'?jid=' + item.id}}">
	    Edit Job Copy</A>
	</TH>
	{% if oh_task.cron_string %}    
	<TH>
	  <A HREF="{{url_for('ox_herd.cancel_job')+'?jid=' + item.id}}">
	    Cancel Job</A>
	</TH>
	{% endif %}
  </TR></TABLE>
  <UL>
    <LI> job id : {{item.id}} </LI>
    {% for name, value in oh_task.get_display_fields() %}
    <LI>
      {{name}} : {{value}}
    </LI>
    {% endfor %}
  </UL>
</details>
