
<div class="footer w3-row-padding content">
  <hr>
  <TABLE cellpadding="12">
    <TR>
      <TH>
	<A HREF="{{url_for('ox_herd.index')}}">Ox Herd</A>
      </TH>
      <TH>
	<A HREF="{{url_for('ox_herd.show_plugins')}}">Show Plugins</A>
      </TH>
      <TH>
	<A HREF="{{url_for('ox_herd.show_scheduled')}}">Show Scheduled</A>
      </TH>
      <TH>
	<A HREF="{{url_for('ox_herd.list_tasks')}}">List Tasks</A>
      </TH>
      <TH>
	<A HREF="{{url_for('ox_herd.show_task_log')}}">Show Task Log</A>
      </TH>
    </TR>
  </TABLE>
</div>
