{% extends "ox_layout.html" %}
{% block body %}

<div class="w3-light-grey w3-margin-bottom content">
  <h2>Task Report</h2>
  <TABLE class="table" cellpadding="10">
    <TH>Name</TH>
    <TH>ID</TH>
    <TH>Status</TH>
    <TH>Result</TH>
    <TH>Finished at</TH>
    <TR>
      <TD>{{ task_data.task_name }}</TD>
      <TD>{{  task_data.task_id  }}</TD>
      <TD>{{ task_data.task_status }}</TD>
      <TD>{{ task_data.return_value }}</TD>
      <TD>{{ task_data.task_end_utc }}</TD>
    </TR>
  </TABLE>
</div>

{% endblock %}
