<!doctype html>
<!-- top level layout.html for ox_herd. -->
<!-- Usually this will be overriden by site layout, but this is -->
<!-- provided in case the main site does not have a layout.html tempalte. -->
<html>
  <head>
    {% block head %}
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="stylesheet" 
	  href="{{ url_for('ox_herd.static', filename='w3.css') }}">   
    <link rel="stylesheet"
	  href="{{ url_for('static', filename='style.css') }}">
    <link rel="stylesheet"
	  href="{{ url_for('static', filename='basic_theme.css') }}">    
    <title>{% block title %} {{ title }} {% endblock %}</title>
    {% endblock %}
  </head>
  <body>
  {% block top_nav %}
  <hr>
  {% with messages = get_flashed_messages() %}
  {% if messages %}
  <ul class=flashes>
    {% for message in messages %}
    <li>{{ message }}</li>
    {% endfor %}
    </ul>
  {% endif %}
  {% endwith %}
  {% endblock %}
  <hr>
  {% block body %}{% endblock %}
  {% block footer %}
  <div class="footer">
    <hr>
    <A HREF="/ox_herd">Ox Herd</A>
  </div>
  {% endblock %}
  </body>
</html>
